#----------------------------------------------------------------------------
#
# SPIdata
# 
# this script reads spectra into XSPEC from a specified type-II PHA spectral
# data file, incorporating the [datagroup, dataset] scheme.
#
# INTEGRAL-specific options are to 1) have all data groups common:
# use "-" as 3rd command line paramter (actually, any ascii character will work)
# 2) group by detector: use "det" as the 3rd (GroupBy) command-line parameter
# or  3)  use "time" to group the data by science window (or dither point
# for typical (non-staring mode) observation strategy
#
# The command line arguments are then used to generate the appropriate sequence
# of   XSPEC> data n:m phafile.pha{i,j}  commands
#
#
#   Inputs:
#            filename       (string) full name of pha-II data file
#            Nspec          (integer) total number of spectra to load with 
#                            sequential  XSPEC> "data n:m"  commands
#            GroupBy        (string) "det", "time" or "-" (default=det, 
#                            group data by detector; time, group by 
#                            dither step (science window), "-" don't group
#            IgnBadChans    (string) Y/N flag; if yes, ignore channels 
#                            corresponding to ~1350-1650keV
#	     Ndet            Number of detectors (default is 19). The detector
#	     		     selection cut should be made at the SPIGTI step
#	    		     of the analysis.For post Dec. 2003, there can be at
#	   		     most 18 detectors, and for post July 2004 at most 17.
#
#            SpecRngLst     (optional) ascii file listing the spectral row-id
#                            ranges in the form (row_i, row_i+1) to use in
#                            the analysis; ie. the rows of the PHA-II data 
#                            file. it is recommended that these consist of 
#                            complete science windows,e.g.a sample file might be:
#                            
#                            1, 19
#                            38, 76
#                            114, 171
#
#
#   Examples:
#		1) read in all SPI spectra from a 5x5 dither (for which the full 
#		   SPI 19-detector array was active). Group the spectra by detector,
#		   and instruct XSPEC to "ignore" the channels between about 
#		   1250-1500 keV where the data are effected by detecter-electronic 
#		   noise):
#
#		XSPEC> source SPIdata.tcl
#		XSPEC> SPIdata myDataDir/myPHA2File.pha 475 det Y 
#			
#		   (note that the first step can be done at XSPEC startup by 
#	            adding "source SPIdata.tcl" to xspec.rc)
#
#
#		2) Similar to above but now only 17 detectors are working:
#
#		XSPEC> SPIdata myDataDir/myPHA2File.pha 475 det Y 17
#
#		3) Similar again to 1, with 19 detectors are working, but in this 
#		case we only want to analyze the first 100 and the last 50 spectra:
#
#		we create an ascii file called mySpecSelection.txt containing the lines:
#		1, 100
#		376, 475
#
#		XSPEC> SPIdata myDataDir/myPHA2File.pha 475 det Y 17 mySpecSelection.txt
#
#
#
#      C.R. Shrader,  Code 661, NASA/GSFC  08/2003
#
#----------------------------------------------------------------------------

proc SPIdata { {filename " "} {Nspec 0} {GroupBy "-"} {IgnBadChans "Y"} {Ndet 19}  
			      {SpecRngLst "NONE"} } { 

    if {$Nspec == 0}  {

      puts "\n SPIdata requires 3 arguments:"
      puts " SPIdata filename Nspec GroupBy\n"

      puts "  filename       (string) full name of pha-II data file "
      puts "  Nspec          (integer) total number of spectra to load with  "
      puts "                  sequential  XSPEC> 'data n:m'  commands "
      puts "  GroupBy        (string) 'det', 'time' or '-' (default=det,  "
      puts "                  group data by detector; time, group by  "
      puts "                  dither step (science window), '-' don't group "
      puts "  IgnBadChans    (string) Y/N flag; if yes, ignore channels "
      puts "                  corresponding to ~1250-1650keV"
      puts "  Ndet            Number of detectors (default is 19). The detector"
      puts "  		      selection cut should be made at the SPIGTI step"
      puts " 		      of the analysis.For post Dec. 2003, there can be at"
      puts "		      most 18 detectors, and for post July 2004 at most 17."
      puts "  SpecRngLst     (optional) ascii file listing the spectral row-id"
      puts "                  ranges in the form (row_i, row_i+1) to use in"
      puts "                  the analysis; ie. the rows of the PHA-II data "
      puts "                  file. it is recommended that these consist of "
      puts "                  complete science windows,e.g.a sample file might be:"
      puts "                  "
      puts "                  1, 19"
      puts "                  38, 76"
      puts "                  114, 171 \n"

      set result " "
      return $result
    }

    set ispec 1
    set j 0; set iDatSet 0; set iDatGrp 1; set l 1

    set   Det       "d"
    set   Time      "t"
    set   noGroup   "-"
    set   blnk      " "
#    set	  SkipDet2   "N"	# this feature has been disabled; make detector cut
#				# when running spigit
#
#  for string arguments: handle case conversions, abbreviations, etc

# this feature has been disabled
#
#    set SkipDet2  [string tolower $SkipDet2]
#    set SkipDet2  [string range $SkipDet2 0 0]

    set GroupBy [string tolower $GroupBy]
    set GroupBy [string range $GroupBy 0 0]

#
#
# save any useful info to a temporary file; theses can be used 
# by SPIuntie, SPIfreeze, etc.

	set DateTime [exec date]
	set SessLog "SPIdata_session.tmp"
	set tmpfil [open $SessLog "w"]

	puts $tmpfil $DateTime
	puts $tmpfil $filename
	puts $tmpfil $Nspec
        puts $tmpfil $Ndet
	puts $tmpfil $GroupBy
	puts $tmpfil $SpecRngLst


# 
# check to see if a file of spectral-row-number intervals is passed or 
# not. if it is, call get_spec_ranges - otherwise, initialze the row-nuber
# index array as [,1,...,-Nspec-1]

    if {$SpecRngLst != "NONE"} {
	get_spec_ranges $SpecRngLst Nsp RowIdx
	set Nspec [expr $Nsp -1]
    }

    if {$SpecRngLst == "NONE"} {
       for {set j 0} {$j < $Nspec} {incr j} {
	   set RowIdx($j)  [expr $j  + 1]
       }
    set j 0
    }

# handle cases where Detector 2 is not to be included in 
# spectral fitting, but it was included in the analysis (i.e. the 
# standard 0-18 detector cut was made)
#
#   if {$SkipDet2 == "Y"} {
#       for {set j 0} {$j < $Nspec} {incr j} {
#	   if {fmod($RowIdx($j),$Ndet) == 3 } {
#	       incr NullSpec 
#	   }
#	   set RowIdx($j) [expr $RowIdx($j) + $NullSpec]
#       }
#
#       set Nspec [expr $Nspec - $NullSpec]
#   }
#
#


#
#   case 1: GroupBy == "-"  data group always = 1
#
    if {$GroupBy == $noGroup} {

	data $filename{$RowIdx(0)-[expr $RowIdx(0)+ $Nspec -1]}
	for {set j 0} {$j < $Nspec} {incr j} {
	    puts $tmpfil " 1  $j  $RowIdx($j)  "
	}
    }

#
#   case 2: GroupBy == "det"  data group repeatedly cycles through 1-Ndet
#
    if {$GroupBy == $Det} {
	set iDatSet 1
	while {$iDatSet <= $Nspec} {
	    set iDatGrp 1
	    while {$iDatGrp <= $Ndet && $ispec <= $Nspec} {
	      set ispec $iDatSet
	      data $iDatGrp:$iDatSet $filename{$RowIdx([expr $ispec - 1])}
	      puts $tmpfil  " $iDatGrp   $iDatSet   $RowIdx([expr $ispec - 1])"
	      incr iDatGrp
	      incr iDatSet
	    }
	}
    }

#
#   case 3: GroupBy == "tim"  data group increase by 1 w/each dither point
#
    if {$GroupBy == $Time} {
	while {$iDatGrp <= $Ndet && $ispec <= $Nspec} {
	 while {$l <= $Ndet  &&  $ispec <= $Nspec}  {
	    set iDatSet [expr $l+$j*$Ndet]
            data $iDatGrp:$iDatSet $filename{$RowIdx([expr $ispec - 1])}
	    puts $tmpfil  " $iDatGrp   $iDatSet   $RowIdx([expr $ispec - 1])"
	     incr ispec;  incr l
      }
 	 incr j; incr iDatGrp;  set l 1
     }
    }
#
# now run (optional) ignore command
#

    set IgnBadChans [string toupper $IgnBadChans]
    if {$IgnBadChans == "Y"} {
	ignore 1-$ispec:1325.0-1685.0
    }

#
# close session log file

	close $tmpfil

#
# finished, returning

    set result "Done, $ispec spectra read"
    return $result

 }