#-------------------------------------------------------------------------------
#
# SPIuntie
# 
# this script takes unties linked paramaters for a data set containing Nspec 
# spectra already in memory
#
# INTEGRAL-specific grouping, should already set with "SPIdata" or equivalent
# sequence of XSPEC "data n:m" commands
#
# Since the "spibkg" model ranks the paramaters of the background (other than 
# the global noramalization) by the a variability-amplittude statistic, the 
# usual appraoch is make a first order fit with linked parameters (i.e. 
# not fixed background over time, detector), then unlinking (or "untie'ing) 
# the individual noramalizatin terms, then the 1st N ranked channels. For 
# example,  XSPEC> SPIuntie 475 det 1 10    will untie the normalizations,
# plus the 9 most highly flucatuating energy channels for subseqeunt fitting
#
#
#   Inputs:
#
#         Pstart       (integer) untie non-normalization parameters starting with 
#                      this one,or set to -1 to untie only normalization terms
#                      to ignore subsequent arguments & untie only Normalization"
#         Pstop        (integer) untie non-normalization parameters finishing with 
#                       this one (def=2) 
#	  inFil	       Name of the SPI session log file (default is 
#		       SPIdata_session.tmp). In most cases, this should just be 
#		       left unspecified, and the default used.
#
#
#    Example:
#	      1) Assume we've read in a few hundred spectra, grouped by detector. 
#		 A first fitting pass has been made, and now we want to decouple 
#		 the (detector-specific) normalization terms of the background model:
#
#		 XSPEC> SPIuntie -1
#
#
#	      2) Similar to the above case, but we now wish to decouple the two 
#		 background model paramters corresponding the highest spectra-to
#		 spectra variance:
#
#		 XSPEC> SPIuntie 1 2
#
#
#      C.R. Shrader,  CCode 661, NASA/GSFC  08/2003
#
#-------------------------------------------------------------------------------

proc SPIuntie {  {Pstart -1} {Pstop 1} {inFil "SPIdata_session.tmp"} }  {


    if { $Pstart == "-h" }  {

      puts "\n SPIuntie requires 1-3 arguments:"
      puts " SPIuntie Pstart Pstop <infil> \n"

      puts " Pstart       (integer) untie non-normalization parameters starting with this one (def=1),"
      puts "              or set to -1 to untie only normalization terms"
      puts "              to ignore subsequent arguments & untie only Normalization"
      puts " Pstop        (integer) untie non-normalization parameters finishing with this one def=2 "
      puts " inFil        (optional, string) name of session log file (defaults to \n"
      puts "              ./SPIdata_sessions.log, which is created by the SPIdata script. Its"
      puts "		  usually best not to specifcy this, and use the (which gets set "
      puts " 		  transparently by SPIdata \n"

	set result " "
	return $result
    }

#
# get available informatin from session log, and tclout

    # open input file (default is "SPIdata_session.tmp" created by SPIdata

    set inFil [open $inFil "r"]

# write number of spectra and number of background model parameters
# to output fil

# new read in line be line, skipping the header stuff

    gets $inFil Date
    gets $inFil inpPHAfil
    gets $inFil Nspec
    gets $inFil Ndet
    gets $inFil GroupBy
#
    close $inFil
#
# now get model ID and name

    set   bkgMdl    "bkg"
    set   spiBkgMdl "spibkg_lo"
#
    getModID bkgMdl spiBkgMdl 
#
    set   ispec 1
    set   j 0
    set   i 0

    set   Det       "det"
    set   Time      "time"
    set   noGroup   "0"
    set   Nparams   10
    set   blnk " "
#
# get available info: Nspec, Grpouping (Det or Point?), model ID and XSPEC model name:

# fix truncations or capitalization problems

    set GroupBy [string tolower $GroupBy]
    set GroupBy [string range $GroupBy 0 0]

#
# set the number of model parameters

    if {$spiBkgMdl == "spibkg_med"} { set Nparams 25 }
    if {$spiBkgMdl == "spibkg_hi"}  { set Nparams 50 }
#
# and a few more defaults (if Pstart is negative, Pstop is not used)

    if {$Pstart < 0} {set Pstop -1}   

    
#
#   case 1: GroupBy == "-", number of data groups always = 1
#
    if { $GroupBy != "-" && $GroupBy != "d" && $GroupBy != "t" } {
  	    puts "\n\n Data Grouping Scheme Not Well Defined (more groups than spectra) "
	    set result " "
	    return $result
    }


#
# Case 2: by detector 
#
    if {$GroupBy == "d"}  {
	set GrpSiz [expr $Nspec/$Ndet]
	set Ngrps  $Ndet
     while {$i <= $Pstop} {
	 while { $j < $Ngrps } {
	   set iParam [expr $i + $j*[expr $Nparams+1] ]
	   puts "\n untie $bkgMdl:$iParam"
           untie $bkgMdl:$iParam
           incr j
           incr ispec
        }
      incr i
      set  j 0
     }
   }

#
# Case 2: by  time (@ 1-dither intervals)
#
    if {$GroupBy == "t"}  {
     set GrpSiz $Ndet
     set Ngrps  [expr $Nspec/$GrpSiz]
     while {$i <= $Pstop} {
	 while { $j < $GrpSiz } {
	   set iParam [expr $i + $j*[expr $Nparams+1] ]
	   puts "\n untie $bkgMdl:$iParam"
           untie $bkgMdl:$iParam
           incr j
           incr ispec
        }
      incr i
      set  j 0
     }
   }

#
# untie Normalizations?
#

#    set Norm [string toupper $Norm]

      set  j 1
    if {$Pstart == [expr $Nparams+1] || $Pstart < 0} {
	  while { $j < $Ngrps } {
	    set iParam [expr [expr $j+1]*[expr $Nparams+1]]
            puts "\n untie $bkgMdl:$iParam"
	    untie $bkgMdl:$iParam
            incr j
            incr ispec
        }
    }

    set result "Done, $ispec spectral parameters untied"
    return $result
}